#include <bios.h>
#include <dos.h>
#include <stdio.h>
#include <graphics.h>
#include <time.h>

char table[256] = { 0, 4, 2, 6, 1, 5, 3, 7, 8, 12, 10, 14, 9, 13, 11, 15 };

set(int y, x, b, bpp) {
    int b0, b1, b2, b3, b4, b5, b6, b7;
    union REGS regs;

    switch (bpp) {
        case 1:
            b0 = (b>>7) & 1;
            b1 = (b>>6) & 1;
            b2 = (b>>5) & 1;
            b3 = (b>>4) & 1;
            b4 = (b>>3) & 1;
            b5 = (b>>2) & 1;
            b6 = (b>>1) & 1;
            b7 = (b>>0) & 1;
            break;
        case 2:
            b0 = (b>>6) & 3;
            b1 = (b>>4) & 3;
            b2 = (b>>2) & 3;
            b3 = (b>>0) & 3;
            break;
        case 4:
            b0 = (b>>4) & 0x0f;
            b1 = (b>>0) & 0x0f;
            break;
        case 8:
            b0 = (b>>0) & 0xff;
            break;
    } /* switch */

    regs.h.ah = 0x0c;           /* write graphics pixel */
    regs.h.al = table[b0];      /* set pixel value */
    regs.h.bh = 0;              /* set video page */
    regs.x.cx = x;              /* set column */
    regs.x.dx = y;              /* set row */
    int86(0x10, &regs, &regs);  /* do it */
    if (bpp==8) return;
    regs.h.al = table[b1];      /* set pixel value */
    regs.x.cx++;                /* increment column */
    int86(0x10, &regs, &regs);  /* do it */
    if (bpp==4) return;
    regs.h.al = table[b2];      /* set pixel value */
    regs.x.cx++;                /* increment column */
    int86(0x10, &regs, &regs);  /* do it */
    regs.h.al = table[b3];      /* set pixel value */
    regs.x.cx++;                /* increment column */
    int86(0x10, &regs, &regs);  /* do it */
    if (bpp==2) return;
    regs.h.al = table[b4];      /* set pixel value */
    regs.x.cx++;                /* increment column */
    int86(0x10, &regs, &regs);  /* do it */
    regs.h.al = table[b5];      /* set pixel value */
    regs.x.cx++;                /* increment column */
    int86(0x10, &regs, &regs);  /* do it */
    regs.h.al = table[b6];      /* set pixel value */
    regs.x.cx++;                /* increment column */
    int86(0x10, &regs, &regs);  /* do it */
    regs.h.al = table[b7];      /* set pixel value */
    regs.x.cx++;                /* increment column */
    int86(0x10, &regs, &regs);  /* do it */
}

main(int argc, char **argv) {
    time_t start_time, now;
    char i, t, solid = PENCOLOR;
    long length, offset, width, height;
    int bitsperpixel, R=0, C=0, r, c, pixperby, j;
    FILE *fp;

    for (j=16; j<256; j++) table[j]=table[j&0x0f];
    fp = fopen("jwb.bmp", "rb");
    if (fp==NULL) return 1;
    fseek(fp, 2l, SEEK_SET);
    fread(&length, sizeof(long), 1, fp);
    fseek(fp, 10l, SEEK_SET);
    fread(&offset, sizeof(long), 1, fp);
    fseek(fp, 18l, SEEK_SET);
    fread(&width, sizeof(long), 1, fp);
    fseek(fp, 22l, SEEK_SET);
    fread(&height, sizeof(long), 1, fp);
    fseek(fp, 28l, SEEK_SET);
    fread(&bitsperpixel, sizeof(int), 1, fp);
    width = width&7?(width+8) & 0xfffffff8:width;
        
    fseek(fp, offset, SEEK_SET);

    switch (bitsperpixel) {
        case 1:
            pixperby = 8;
            setvmode(6);
            table[1] = 1;
            R = 100-height/2;
            C = 320-width/2;
            break;
        case 2:
            table[1]=1;
            table[2]=2;
            table[3]=3;
            pixperby = 4;
            setvmode(4);
            R = 100-height/2;
            C = 160-width/2;
            break;
        case 4:
            pixperby = 2;
            if ((t = setvmode(18))==18) {
                R = 240-height/2;
                C = 320-width/2;
            } else {
                setvmode(14);
                R = 100-height/2;
                C = 320-width/2;
            } /* if else */
            break;
        case 8:
            pixperby = 1;
            setvmode(19);
            R = 100-height/2;
            C = 160-width/2;
            break;
    } /* switch */
            
    /* if (width & 1) width++;    */
    for (r=0; r<height; r++)
        for (c=0; c<width; c+=pixperby) {
            fread(&i, 1, 1, fp);
            set(R+height-r, C+c, i, bitsperpixel);
    }   
    time(&start_time);
    now = start_time;
    while ((difftime(now, start_time)<10)&&!bioskey(1)) time(&now);
    if (bioskey(1)) bioskey(0);
    setvmode(3);
    execl("addrbook.pgm",NULL);
}

