#ifndef   SCREEN_H
#define   SCREEN_H
#include  <bios.h>
#include  <dos.h>
#include  <stdio.h>
#include  <stdlib.h>
#include  <time.h>

#define   _screen_default_attribute     0x07
#define   _screen_default_width         80
#define   _screen_default_page          0
#define   _screen_default_mode          7

#define   MONO                          0xb0000000
#define   COLOR                         0xb8000000

int  _screen_attribute        =         _screen_default_attribute;
int  _screen_width            =         _screen_default_width;
int  _screen_page             =         _screen_default_page;
int  _screen_mode             =         _screen_default_mode;

union REGS inregs, outregs;

#ifndef __POWERC
int  getvmode(void);
int  setvmode(int mode);
int  getvpage(void);
int  setvpage(int page);
int  getvwidth(void);
int  cursrow(void);
int  curscol(void);
void curson(void);
void cursoff(void);
void cursblk(void);
void curslin(void);
void curstype(int curs_start_line, int curs_end_line);
void poscurs(int row, int column);
void clrscrn(void);
void clrscrn2(int attr);
void clearwindow(int up, int lft, int dn, int rt, int attr);
void windowscrollup(int up, int lft, int dn, int rt, int attr, int n);
void windowscrolldown(int up, int lft, int dn, int rt, int attr, int n);
void Sound(int frequency, int duration);
void Sleep(clock_t wait);
void pause(int secs);
FILE *savescreen(void);
void loadscreen(FILE *fp);

int  setvmode(int mode) {
     inregs.h.ah = 0;
     inregs.h.al = mode;
     int86(0x10, & inregs, & outregs);
     return getvmode();
} /* setvmode */

int  getvmode(void) {
     getvwidth();

     return _screen_mode;
} /* getvmode */

int  cursrow(void) {
     getvwidth();
     inregs.h.ah = 0x03;
     inregs.h.bh = _screen_page;
     int86(0x10, &inregs, &outregs);

     return outregs.h.dh;
} /* cursrow */

int  curscol(void) {
     getvwidth();
     inregs.h.ah = 0x03;
     inregs.h.bh = _screen_page;
     int86(0x10, &inregs, &outregs);

     return outregs.h.dl;
} /* curscol */

void clrscrn(void) {
     getvwidth();
     clearwindow(0, 0, 24, _screen_width-1, _screen_attribute);
} /* clrscrn */

void clrscrn2(int attr) {
     clearwindow(0, 0, 24, _screen_width-1, attr);
} /* clrscrn2 */

void poscurs(int row, int column) {
     getvwidth();
     inregs.h.ah = 0x02;
     inregs.h.bh = _screen_page;
     inregs.h.dh = row;
     inregs.h.dl = column;
     int86(0x10, & inregs, & outregs);
} /* poscurs */

void curson(void) {
     curslin();
} /* curson */

void cursoff(void) {
     curstype(0x20, 7);
} /* cursoff */

void curslin(void) {
     getvwidth();
     if (_screen_mode == 7)
          curstype(11, 12);
     else
          curstype(6, 7);
} /* curslin */

void cursblk(void) {
     getvwidth();
     if (_screen_mode == 7)
          curstype(0, 12);
     else
          curstype(0, 7);
} /* cursblk */

#else

int  getvpage(void);
int  setvpage(int page);
int  getvwidth(void);
void curstype(int curs_start_line, int curs_end_line);
void clearwindow(int up, int lft, int dn, int rt, int attr);
void windowscrollup(int up, int lft, int dn, int rt, int attr, int n);
void windowscrolldown(int up, int lft, int dn, int rt, int attr, int n);
void Sound(int frequency, int duration);
void Sleep(clock_t wait);
void pause(int secs);
FILE *savescreen(void);
void loadscreen(FILE *fp);

#endif

int  getvpage(void) {
     getvwidth();

     return _screen_page;
} /* getvpage */

int  getvwidth(void) {
     inregs.h.ah = 0x0f;
     int86(0x10, &inregs, &outregs);
     _screen_width = outregs.h.ah;
     _screen_page  = outregs.h.bh;
     _screen_mode  = outregs.h.al;

     return _screen_width;
} /* getvwidth */

int  setvpage(int page) {
     getvwidth();
     inregs.h.ah = 0x05;
     inregs.h.al = page;
     int86(0x10, &inregs, &outregs);
     getvwidth();

     return _screen_page;
} /* setvpage */

void curstype(int curs_start_line, int curs_end_line) {
     inregs.h.ah = 0x01;
     inregs.h.ch = curs_start_line;
     inregs.h.cl = curs_end_line;
     int86(0x10, &inregs, &outregs);
} /* curstype */

void clearwindow(int up, int lft, int dn, int rt, int attr) {
     getvwidth();
     inregs.x.ax = 0x0600;
     inregs.h.bh = attr;
     inregs.h.ch = up;
     inregs.h.cl = lft;
     inregs.h.dh = dn;
     inregs.h.dl = rt;
     int86(0x10, &inregs, &outregs);
     getvwidth();
     poscurs(up, lft);
} /* clearwindow */

void windowscrollup(int up, int lft, int dn, int rt, int attr, int n) {
     getvwidth();
     inregs.h.ah = 0x06;
     inregs.h.al = n;
     inregs.h.bh = attr;
     inregs.h.ch = up;
     inregs.h.cl = lft;
     inregs.h.dh = dn;
     inregs.h.dl = rt;
     int86(0x10, &inregs, &outregs);
     getvwidth();
     poscurs(dn-n+1, lft);
} /* windowscrollup */

void windowscrolldown(int up, int lft, int dn, int rt, int attr, int n) {
     getvwidth();
     inregs.h.ah = 0x07;
     inregs.h.al = n;
     inregs.h.bh = attr;
     inregs.h.ch = up;
     inregs.h.cl = lft;
     inregs.h.dh = dn;
     inregs.h.dl = rt;
     int86(0x10, &inregs, &outregs);
     getvwidth();
     poscurs(up, lft);
} /* windowscrolldown */

/* Sounds the speaker for a time specified in microseconds by duration
 * at a pitch specified in hertz by frequency.
 */
void Sound( int frequency, int duration )
{
     int control;

     /* If frequency is 0, Sound doesn't try to make a Sound. It
     * just Sleeps for the duration.
     */
     if( frequency )
     {
          /* 75 is about the shortest reliable duration of a Sound. */
          if( duration < 75 )
               duration = 75;

          /* Prepare timer by sending 10111100 to port 43. */
          outp( 0x43, 0xb6 );

          /* Divide input frequency by timer ticks per second and
          * write (byte by byte) to timer.
          */
          frequency = (unsigned)(1193180L / frequency);
          outp( 0x42, (char)frequency );
          outp( 0x42, (char)(frequency >> 8) );

          /* Save speaker control byte. */
          control = inp( 0x61 );

          /* Turn on the speaker (with bits 0 and 1). */
          outp( 0x61, control | 0x3 );
     }

     Sleep( (clock_t)duration );

     /* Turn speaker back on if necessary. */
     if( frequency )
          outp( 0x61, control );

} /* Sound */

/* Pauses for a specified number of microseconds. */
void Sleep( clock_t wait )
{
     clock_t goal;

     goal = wait + clock();
     while( goal > clock() )
          if (kbhit()) {
               getch();
               break;
          }
} /* Sleep */

void pause( int secs) {
     Sleep(1000 * secs);
} /* pause */

FILE *savescreen(void)
{
     char far *video;
     int mode, i, size;

     FILE *fp;

     mode = getvmode();
     if (mode == 7) video = MONO; else video = COLOR;
     switch (mode) {
         case 0:
         case 1: size = 2000; break;
         case 2:
         case 3:
         case 7: size = 4000; break;
         default: size = 0; break;
     } /* end switch */

     fp = tmpfile();
     if (size == 0) return(NULL);
     putc(mode, fp);
     for (i=0; i<size; i++) putc(*video++, fp);

     return(fp);
}

void loadscreen(FILE *fp)
{
     char far *video;
     int i, mode, size;

     if (fp == NULL) return;
     rewind(fp);
     mode = getc(fp);
    
     if (mode == 7) video = MONO; else video = COLOR;
     switch (mode) {
         case 0:
         case 1: size = 2000; break;
         case 2:
         case 3:
         case 7: size = 4000; break;
         default: size = 0; break;
     }

     setvmode(mode);
     for (i=0; i<size; i++) *video++ = getc(fp);
     fclose(fp);
}
#undef    COLOR
#undef    MONO

#endif

